package service

import (
	"reflect"

	cda "git.loafle.net/commons_go/di/annotation"
	cdr "git.loafle.net/commons_go/di/registry"
	"git.loafle.net/commons_go/logging"
	ooccp "git.loafle.net/overflow/overflow_commons_go/config/probe"
	discoveryM "git.loafle.net/overflow/overflow_commons_go/modules/discovery/model"
	oocmp "git.loafle.net/overflow/overflow_commons_go/modules/probe"
)

func init() {
	cdr.RegisterType(reflect.TypeOf((*DiscoveryService)(nil)))
}

type DiscoveryService struct {
	cda.TypeAnnotation `annotation:"@overFlow:Service()"`

	ContainerService *ContainerService `annotation:"@Inject()"`
	CentralService   *CentralService   `annotation:"@Inject()"`
}

func (ds *DiscoveryService) DiscoverZone(requesterID string, dz *discoveryM.DiscoveryZone) error {
	logging.Logger().Debugf("DiscoverZone: requesterID(%s)", requesterID)

	return ds.ContainerService.Send(ooccp.ContainerDiscoveryName, "DiscoveryService.DiscoverZone", requesterID, dz)
}

func (ds *DiscoveryService) DiscoverHost(requesterID string, zone *discoveryM.Zone, dh *discoveryM.DiscoveryHost) error {
	return ds.ContainerService.Send(ooccp.ContainerDiscoveryName, "DiscoveryService.DiscoverHost", requesterID, zone, dh)
}

func (ds *DiscoveryService) DiscoverPort(requesterID string, host *discoveryM.Host, dp *discoveryM.DiscoveryPort) error {
	return ds.ContainerService.Send(ooccp.ContainerDiscoveryName, "DiscoveryService.DiscoverPort", requesterID, host, dp)
}

func (ds *DiscoveryService) DiscoverService(requesterID string, port *discoveryM.Port, dService *discoveryM.DiscoveryService) error {
	return ds.ContainerService.Send(ooccp.ContainerDiscoveryName, "DiscoveryService.DiscoverZone", requesterID, port, dService)
}

// use by discovery
func (ds *DiscoveryService) DiscoveredZone(requesterID string, zone *discoveryM.Zone) error {
	logging.Logger().Debugf("DiscoveredZone: requesterID(%s) network(%s) ip(%s) mac(%s) iface(%s)", requesterID, zone.Network, zone.IP, zone.Mac, zone.Iface)
	return ds.CentralService.Send(oocmp.HTTPEntry_Probe, "DiscoveryService.discoveredZone", requesterID, zone)
}

func (ds *DiscoveryService) DiscoveredHost(requesterID string, host *discoveryM.Host) error {
	return ds.CentralService.Send(oocmp.HTTPEntry_Probe, "DiscoveryService.discoveredHost", requesterID, host)
}

func (ds *DiscoveryService) DiscoveredPort(requesterID string, port *discoveryM.Port) error {
	return ds.CentralService.Send(oocmp.HTTPEntry_Probe, "DiscoveryService.discoveredPort", requesterID, port)
}

func (ds *DiscoveryService) DiscoveredService(requesterID string, service *discoveryM.Service) error {
	return ds.CentralService.Send(oocmp.HTTPEntry_Probe, "DiscoveryService.discoveredService", requesterID, service)
}
